@@ -1,5 +1,5 @@
 /*
- * ejabberd, Copyright (C) 2002-2013   ProcessOne
+ * ejabberd, Copyright (C) 2002-2015   ProcessOne
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -499,14 +499,10 @@ static ErlDrvSSizeT tls_drv_control(ErlDrvData handle,
 	    SSL_CTX_set_cipher_list(ctx, ciphers);
 
 #ifndef OPENSSL_NO_ECDH
-	    if (command == SET_CERTIFICATE_FILE_ACCEPT) {
-		setup_ecdh(ctx);
-	    }
+	    setup_ecdh(ctx);
 #endif
 #ifndef OPENSSL_NO_DH
-	    if (command == SET_CERTIFICATE_FILE_ACCEPT) {
-		setup_dh(ctx);
-	    }
+	    setup_dh(ctx);
 #endif
 
 	    SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
@@ -729,6 +725,7 @@ ErlDrvEntry tls_driver_entry = {
 
 DRIVER_INIT(p1_tls_drv) /* must match name in driver_entry */
 {
+   CRYPTO_set_mem_functions(driver_alloc, driver_realloc, driver_free);
    OpenSSL_add_ssl_algorithms();
    SSL_load_error_strings();
    init_hash_table();
